/*
 * $Header: /var/lib/cvs/AirportEditor/src/com/greghawkes/flightgear/tools/airporteditor/Parking.java,v 1.1.1.1 2008-06-04 13:59:36 gregh Exp $
 * Copyright 2008 Gregory D. Hawkes.
 *
 * This file is part of AirportEditor.
 *
 * AirportEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * AirportEditor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AirportEditor.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.greghawkes.flightgear.tools.airporteditor;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.greghawkes.flightgear.GroundObject;
import com.greghawkes.utils.CoordUtils;
import com.greghawkes.utils.DateTimeUtils;
import com.greghawkes.utils.LinearInterpolator;
import com.greghawkes.utils.NumberUtils;
import com.greghawkes.utils.StringUtils;

/**
 * Reads and writes airport data from a FlightGear parking.xml file.
 *
 * @author $Author: gregh $
 * @version $Id: Parking.java,v 1.1.1.1 2008-06-04 13:59:36 gregh Exp $
 */
public class Parking {
    private static final String XML_TRUE = "1";
    private static final String XML_FALSE = "0";

    private final Airport airport;
    private final LinearInterpolator latitude;
    private final LinearInterpolator longitude;

    /**
     * Construct a new Parking object.
     * @param airport the airport to be processed.
     * @param latitude converts latitude coordinates.
     * @param longitude converts longitude coordinates.
     */
    public Parking(Airport airport, LinearInterpolator latitude, LinearInterpolator longitude) {
        this.airport = airport;
        this.latitude = latitude;
        this.longitude = longitude;
    }

    /**
     * Write the airport's taxiways to a FlightGear parking.xml file.
     *
     * @param pw the file to write.
     */
    public void writeParking(PrintWriter pw) {
        List<TerminalGate> taxiGates = new ArrayList<TerminalGate>();
        List<TaxiNode> taxiNodes = new ArrayList<TaxiNode>();

        for (TaxiNode n : airport.getNodes()) {
            if (n instanceof TerminalGate) {
                taxiGates.add((TerminalGate) n);

            } else {
                taxiNodes.add(n);
            }
        }

        Date now = new Date();
        pw.println("<?xml version=\"1.0\" ?>");
        pw.println();
        pw.println("<!--");
        pw.println("Flightgear parking.xml file for " + airport.getDescription() + " (" + airport.getIcaoCode() + ")");
        pw.println("Generated by " + AirportEditor.getApplication().getClass().getName());
        pw.println("Created by " + System.getProperty("user.name") + " on " + DateTimeUtils.getDateTimeLabel(now));
        pw.println("-->");
        pw.print("<groundnet");
        pw.print(" icaoCode=\"" + StringUtils.escapeXML(airport.getIcaoCode()) + "\"");
        pw.print(" dateUpdated=\"" + DateTimeUtils.isoDateTime(now) + "\"");
        pw.println(">");

        int ident = 0;
        pw.println("\t<parkinglist>");
        for (TerminalGate n : taxiGates) {
            n.setIdent(ident);
            double lat = latitude.valueAt(n.getY());
            double lon = longitude.valueAt(n.getX());

            pw.print("\t\t<Parking");
            pw.print(" index=\"" + ident + "\"");
            pw.print(" type=\"" + StringUtils.escapeXML(n.getGateType().toLowerCase()) + "\"");
            pw.print(" name=\"" + StringUtils.escapeXML(n.getNodeName()) + "\"");
            pw.print(" number=\"" + StringUtils.escapeXML(n.getNumber()) + "\"");
            pw.print(" lat=\"" + StringUtils.escapeXML(CoordUtils.formatLatDegMin(lat)) + "\"");
            pw.print(" lon=\"" + StringUtils.escapeXML(CoordUtils.formatLonDegMin(lon)) + "\"");
            pw.print(" heading=\"" + ((int) n.getHeading()) + "\"");
            pw.print(" radius=\"" + n.getGateSize() + "\"");
            pw.print(" airlineCodes=\"" + StringUtils.escapeXML(n.getAirlineCodes()) + "\"");
            pw.print(" pushBack=\"left\"");
            pw.println(" />");
            ident++;
        }
        pw.println("\t</parkinglist>");

        pw.println("\t<TaxiNodes>");
        for (TaxiNode n : taxiNodes) {
            n.setIdent(ident);
            double lat = latitude.valueAt(n.getY());
            double lon = longitude.valueAt(n.getX());

            pw.print("\t\t<node");
            pw.print(" index=\"" + ident + "\"");
            pw.print(" lat=\"" + StringUtils.escapeXML(CoordUtils.formatLatDegMin(lat)) + "\"");
            pw.print(" lon=\"" + StringUtils.escapeXML(CoordUtils.formatLonDegMin(lon)) + "\"");
            pw.print(" isOnRunway=\"" + (n.isOnRunway() ? XML_TRUE : XML_FALSE) + "\"");
            pw.print(" holdPointType=\"" + StringUtils.escapeXML(n.getHoldPointType()) + "\"");
            pw.print(" name=\"" + StringUtils.escapeXML(n.getNodeName()) + "\"");
            pw.println(" />");
            ident++;
        }
        pw.println("\t</TaxiNodes>");

        pw.println("\t<TaxiWaySegments>");
        for (TaxiPath s : airport.getTaxiway()) {
            int dir = s.getDirection();

            if (dir == TaxiPath.FORWARD || dir == TaxiPath.BOTH) {
                pw.print("\t\t<arc");
                pw.print(" begin=\"" + s.getStart().getIdent() + "\"");
                pw.print(" end=\"" + s.getFinish().getIdent() + "\"");
                pw.print(" isPushBackRoute=\"" + (s.isPushBackRoute() ? XML_TRUE : XML_FALSE) + "\"");
                pw.print(" name=\"Route\"");
                pw.println(" />");
            }
            if (dir == TaxiPath.BACKWARD || dir == TaxiPath.BOTH) {
                pw.print("\t\t<arc");
                pw.print(" begin=\"" + s.getFinish().getIdent() + "\"");
                pw.print(" end=\"" + s.getStart().getIdent() + "\"");
                pw.print(" isPushBackRoute=\"" + (s.isPushBackRoute() ? XML_TRUE : XML_FALSE) + "\"");
                pw.print(" name=\"Route\"");
                pw.println(" />");
            }
        }
        pw.println("\t</TaxiWaySegments>");

        pw.println("</groundnet>");
    }

    /**
     * Parse a FlightGear parking.xml file, add it to the airport.
     * @param inf the parking.xml file to parse.
     * @throws org.xml.sax.SAXException if the file cannot be parsed.
     * @throws java.io.IOException if the file cannot be read.
     * @throws javax.xml.parsers.ParserConfigurationException if the parser is not configured correctly.
     */
    public void parseParking(File inf) throws SAXException, IOException, ParserConfigurationException {
        for (TaxiNode node : airport.getNodes()) {
            node.setIdent(TaxiNode.IDENT_UNKNOWN);
        }

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document doc = db.parse(inf);
        doc.getDocumentElement().normalize();
        parseGates(doc.getElementsByTagName("Parking"));
        parseNodes(doc.getElementsByTagName("node"));
        parseSegments(doc.getElementsByTagName("arc"));
    }

    /**
     * Parse the "Parking" element of FlightGear's parking.xml file.
     * @param gates the list of terminal gate nodes.
     */
    private void parseGates(NodeList gates) {
        int count = 0;
        for (int i = 0; i < gates.getLength(); i++) {
            Node n = gates.item(i);
            NamedNodeMap attr = n.getAttributes();

            Double lat = CoordUtils.parse(attr.getNamedItem("lat").getNodeValue());
            Double lon = CoordUtils.parse(attr.getNamedItem("lon").getNodeValue());

            TerminalGate gate = new TerminalGate();
            gate.setIdent(NumberUtils.parseInteger(attr.getNamedItem("index").getNodeValue(), GroundObject.IDENT_UNKNOWN));
            gate.setGateType(attr.getNamedItem("type").getNodeValue());
            gate.setNodeName(attr.getNamedItem("name").getNodeValue());
            gate.setNumber(attr.getNamedItem("number").getNodeValue());
            gate.setHeading(NumberUtils.parseDouble(attr.getNamedItem("heading").getNodeValue(), 0.0));
            gate.setGateSize(NumberUtils.parseInteger(attr.getNamedItem("radius").getNodeValue(), TerminalGate.DEFAULT_SIZE));
            gate.setAirlineCodes(attr.getNamedItem("airlineCodes").getNodeValue());
//            gate.setPushBack(attr.getNamedItem("pushBack").getNodeValue()));
            gate.setY(latitude.valueAt(lat));
            gate.setX(longitude.valueAt(lon));
            airport.addNode(gate);
            count++;
        }
    }

    private void parseNodes(NodeList nodes) {
        int count = 0;
        for (int i = 0; i < nodes.getLength(); i++) {
            Node n = nodes.item(i);
            NamedNodeMap attr = n.getAttributes();

            Double lat = CoordUtils.parse(attr.getNamedItem("lat").getNodeValue());
            Double lon = CoordUtils.parse(attr.getNamedItem("lon").getNodeValue());

            TaxiNode node = new TaxiNode();
            node.setIdent(NumberUtils.parseInteger(attr.getNamedItem("index").getNodeValue(), GroundObject.IDENT_UNKNOWN));
            node.setNodeName(attr.getNamedItem("name").getNodeValue());
            node.setY(latitude.valueAt(lat));
            node.setX(longitude.valueAt(lon));
            airport.addNode(node);
            count++;
        }
    }

    private void parseSegments(NodeList segments) {
        int count = 0;
        for (int i = 0; i < segments.getLength(); i++) {
            Node segment = segments.item(i);
            NamedNodeMap attr = segment.getAttributes();

            int fromIdent = Integer.parseInt(attr.getNamedItem("begin").getNodeValue());
            int toIdent = Integer.parseInt(attr.getNamedItem("end").getNodeValue());
            TaxiNode fromNode = airport.findNode(fromIdent);
            TaxiNode toNode = airport.findNode(toIdent);

            airport.addTaxiway(fromNode, toNode);
            count++;
        }
    }

}

// ----- Revision History -----
// $Log: Parking.java,v $
// Revision 1.1.1.1  2008-06-04 13:59:36  gregh
// File imported into CVS repository
//
